# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['langchain',
 'langchain.agents',
 'langchain.agents.agent_toolkits',
 'langchain.agents.agent_toolkits.csv',
 'langchain.agents.agent_toolkits.json',
 'langchain.agents.agent_toolkits.openapi',
 'langchain.agents.agent_toolkits.pandas',
 'langchain.agents.agent_toolkits.python',
 'langchain.agents.agent_toolkits.sql',
 'langchain.agents.agent_toolkits.vectorstore',
 'langchain.agents.agent_toolkits.zapier',
 'langchain.agents.chat',
 'langchain.agents.conversational',
 'langchain.agents.conversational_chat',
 'langchain.agents.mrkl',
 'langchain.agents.react',
 'langchain.agents.self_ask_with_search',
 'langchain.callbacks',
 'langchain.callbacks.tracers',
 'langchain.chains',
 'langchain.chains.api',
 'langchain.chains.api.openapi',
 'langchain.chains.chat_vector_db',
 'langchain.chains.combine_documents',
 'langchain.chains.constitutional_ai',
 'langchain.chains.conversation',
 'langchain.chains.conversational_retrieval',
 'langchain.chains.graph_qa',
 'langchain.chains.hyde',
 'langchain.chains.llm_bash',
 'langchain.chains.llm_checker',
 'langchain.chains.llm_math',
 'langchain.chains.llm_summarization_checker',
 'langchain.chains.natbot',
 'langchain.chains.pal',
 'langchain.chains.qa_generation',
 'langchain.chains.qa_with_sources',
 'langchain.chains.question_answering',
 'langchain.chains.retrieval_qa',
 'langchain.chains.sql_database',
 'langchain.chains.summarize',
 'langchain.chat_models',
 'langchain.docstore',
 'langchain.document_loaders',
 'langchain.embeddings',
 'langchain.evaluation',
 'langchain.evaluation.qa',
 'langchain.graphs',
 'langchain.indexes',
 'langchain.indexes.prompts',
 'langchain.llms',
 'langchain.memory',
 'langchain.memory.chat_message_histories',
 'langchain.output_parsers',
 'langchain.prompts',
 'langchain.prompts.example_selector',
 'langchain.retrievers',
 'langchain.tools',
 'langchain.tools.bing_search',
 'langchain.tools.google_search',
 'langchain.tools.human',
 'langchain.tools.interaction',
 'langchain.tools.json',
 'langchain.tools.openapi',
 'langchain.tools.openapi.utils',
 'langchain.tools.openweathermap',
 'langchain.tools.python',
 'langchain.tools.requests',
 'langchain.tools.searx_search',
 'langchain.tools.sql_database',
 'langchain.tools.vectorstore',
 'langchain.tools.wikipedia',
 'langchain.tools.wolfram_alpha',
 'langchain.tools.zapier',
 'langchain.utilities',
 'langchain.vectorstores']

package_data = \
{'': ['*'], 'langchain.chains.llm_summarization_checker': ['prompts/*']}

install_requires = \
['PyYAML>=5.4.1',
 'SQLAlchemy>=1,<2',
 'aiohttp>=3.8.3,<4.0.0',
 'dataclasses-json>=0.5.7,<0.6.0',
 'numpy>=1,<2',
 'openapi-schema-pydantic>=1.2,<2.0',
 'pydantic>=1,<2',
 'requests>=2,<3',
 'tenacity>=8.1.0,<9.0.0']

extras_require = \
{'all': ['faiss-cpu>=1,<2',
         'wikipedia>=1,<2',
         'elasticsearch>=8,<9',
         'opensearch-py>=2.0.0,<3.0.0',
         'redis>=4,<5',
         'manifest-ml>=0.0.1,<0.0.2',
         'spacy>=3,<4',
         'nltk>=3,<4',
         'transformers>=4,<5',
         'beautifulsoup4>=4,<5',
         'torch>=1,<2',
         'jinja2>=3,<4',
         'pinecone-client>=2,<3',
         'weaviate-client>=3,<4',
         'google-api-python-client==2.70.0',
         'wolframalpha==5.0.0',
         'anthropic>=0.2.4,<0.3.0',
         'cohere>=3,<4',
         'openai>=0,<1',
         'nlpcloud>=1,<2',
         'nomic>=1.0.43,<2.0.0',
         'huggingface_hub>=0,<1',
         'jina>=3.14,<4.0',
         'google-search-results>=2,<3',
         'sentence-transformers>=2,<3',
         'pypdf>=3.4.0,<4.0.0',
         'networkx>=2.6.3,<3.0.0',
         'aleph-alpha-client>=2.15.0,<3.0.0',
         'deeplake>=3.2.9,<4.0.0',
         'pgvector>=0.1.6,<0.2.0',
         'psycopg2-binary>=2.9.5,<3.0.0',
         'boto3>=1.26.96,<2.0.0',
         'pyowm>=3.3.0,<4.0.0'],
 'all:python_full_version >= "3.8.1" and python_version < "3.12"': ['qdrant-client>=1.1.1,<2.0.0'],
 'all:python_version >= "3.10" and python_version < "3.12"': ['tensorflow-text>=2.11.0,<3.0.0'],
 'all:python_version >= "3.9" and python_version < "4.0"': ['tiktoken>=0.3.2,<0.4.0'],
 'cohere': ['cohere>=3,<4'],
 'llms': ['manifest-ml>=0.0.1,<0.0.2',
          'transformers>=4,<5',
          'torch>=1,<2',
          'anthropic>=0.2.4,<0.3.0',
          'cohere>=3,<4',
          'openai>=0,<1',
          'nlpcloud>=1,<2',
          'huggingface_hub>=0,<1'],
 'openai': ['openai>=0,<1'],
 'qdrant:python_full_version >= "3.8.1" and python_version < "3.12"': ['qdrant-client>=1.1.1,<2.0.0']}

entry_points = \
{'console_scripts': ['langchain-server = langchain.server:main']}

setup_kwargs = {
    'name': 'langchain',
    'version': '0.0.133',
    'description': 'Building applications with LLMs through composability',
    'long_description': "# 🦜️🔗 LangChain\n\n⚡ Building applications with LLMs through composability ⚡\n\n[![lint](https://github.com/hwchase17/langchain/actions/workflows/lint.yml/badge.svg)](https://github.com/hwchase17/langchain/actions/workflows/lint.yml) [![test](https://github.com/hwchase17/langchain/actions/workflows/test.yml/badge.svg)](https://github.com/hwchase17/langchain/actions/workflows/test.yml) [![linkcheck](https://github.com/hwchase17/langchain/actions/workflows/linkcheck.yml/badge.svg)](https://github.com/hwchase17/langchain/actions/workflows/linkcheck.yml) [![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT) [![Twitter](https://img.shields.io/twitter/url/https/twitter.com/langchainai.svg?style=social&label=Follow%20%40LangChainAI)](https://twitter.com/langchainai) [![](https://dcbadge.vercel.app/api/server/6adMQxSpJS?compact=true&style=flat)](https://discord.gg/6adMQxSpJS)\n\n**Production Support:** As you move your LangChains into production, we'd love to offer more comprehensive support.\nPlease fill out [this form](https://forms.gle/57d8AmXBYp8PP8tZA) and we'll set up a dedicated support Slack channel.\n\n## Quick Install\n\n`pip install langchain`\n\n## 🤔 What is this?\n\nLarge language models (LLMs) are emerging as a transformative technology, enabling\ndevelopers to build applications that they previously could not.\nBut using these LLMs in isolation is often not enough to\ncreate a truly powerful app - the real power comes when you can combine them with other sources of computation or knowledge.\n\nThis library is aimed at assisting in the development of those types of applications. Common examples of these types of applications include:\n\n**❓ Question Answering over specific documents**\n\n- [Documentation](https://langchain.readthedocs.io/en/latest/use_cases/question_answering.html)\n- End-to-end Example: [Question Answering over Notion Database](https://github.com/hwchase17/notion-qa)\n\n**💬 Chatbots**\n\n- [Documentation](https://langchain.readthedocs.io/en/latest/use_cases/chatbots.html)\n- End-to-end Example: [Chat-LangChain](https://github.com/hwchase17/chat-langchain)\n\n**🤖 Agents**\n\n- [Documentation](https://langchain.readthedocs.io/en/latest/modules/agents.html)\n- End-to-end Example: [GPT+WolframAlpha](https://huggingface.co/spaces/JavaFXpert/Chat-GPT-LangChain)\n\n## 📖 Documentation\n\nPlease see [here](https://langchain.readthedocs.io/en/latest/?) for full documentation on:\n\n- Getting started (installation, setting up the environment, simple examples)\n- How-To examples (demos, integrations, helper functions)\n- Reference (full API docs)\n- Resources (high-level explanation of core concepts)\n\n## 🚀 What can this help with?\n\nThere are six main areas that LangChain is designed to help with.\nThese are, in increasing order of complexity:\n\n**📃 LLMs and Prompts:**\n\nThis includes prompt management, prompt optimization, generic interface for all LLMs, and common utilities for working with LLMs.\n\n**🔗 Chains:**\n\nChains go beyond just a single LLM call, and are sequences of calls (whether to an LLM or a different utility). LangChain provides a standard interface for chains, lots of integrations with other tools, and end-to-end chains for common applications.\n\n**📚 Data Augmented Generation:**\n\nData Augmented Generation involves specific types of chains that first interact with an external datasource to fetch data to use in the generation step. Examples of this include summarization of long pieces of text and question/answering over specific data sources.\n\n**🤖 Agents:**\n\nAgents involve an LLM making decisions about which Actions to take, taking that Action, seeing an Observation, and repeating that until done. LangChain provides a standard interface for agents, a selection of agents to choose from, and examples of end to end agents.\n\n**🧠 Memory:**\n\nMemory is the concept of persisting state between calls of a chain/agent. LangChain provides a standard interface for memory, a collection of memory implementations, and examples of chains/agents that use memory.\n\n**🧐 Evaluation:**\n\n[BETA] Generative models are notoriously hard to evaluate with traditional metrics. One new way of evaluating them is using language models themselves to do the evaluation. LangChain provides some prompts/chains for assisting in this.\n\nFor more information on these concepts, please see our [full documentation](https://langchain.readthedocs.io/en/latest/?).\n\n## 💁 Contributing\n\nAs an open source project in a rapidly developing field, we are extremely open to contributions, whether it be in the form of a new feature, improved infra, or better documentation.\n\nFor detailed information on how to contribute, see [here](.github/CONTRIBUTING.md).\n",
    'author': 'None',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://www.github.com/hwchase17/langchain',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0',
}


setup(**setup_kwargs)
